#!/usr/bin/env python2.7

from optparse import OptionParser
import sys

class BaseCheckCommand(object):
    OK_EXIT=0
    WARNING_EXIT=1
    CRITICAL_EXIT=2
    UNKNOWN_EXIT=3

    def __init__(self, **kwargs):
        self.params = kwargs

        # Set or initialize an OptionParser instance, raise if set is not OptionParser
        self.parser = self.params.get('parser') or OptionParser()
        assert isinstance(self.parser, OptionParser),\
                "Passed parser is not an instance of OptionParser"

        # Add default options for threshold values
        self.parser.add_option('-w', '--warning',
                                dest='warning',
                                action='store',
                                help='Sets the value for the warning threshold.',
                                metavar='WARNING_THERSHOLD')

        self.parser.add_option('-c', '--critical',
                                dest='critical',
                                action='store',
                                help='Sets the value for the critical threshold.',
                                metavar='CRITICAL_THRESHOLD')

        self.status    = None
        self.perf_data = None

    def usage(self, exit_code=0, msg=''):
        self.parser.print_help()
        if msg:
            print >> sys.stderr, msg
        sys.exit(exit_code)

    def parse_args(self):
        opts, args = self.parser.parse_args()
        if opts.warning == None or opts.critical == None:
            self.usage(exit_code=127, msg='Warning or Critical threshold not set.')
        if len(args) == 0 or len(args) >= 2:
            self.usage(exit_code=127, msg='Invalid number of collectors passed as arguments.')
        return [opts, args]

    def exit(self, err=None, exit_code=OK_EXIT):
        if err:
            print >> sys.stderr, err.message
            sys.exit(self.UNKNOWN_EXIT)
        else:
            print("%s | %s") % (self.status, self.perf_data)
            sys.exit(exit_code)



#!/usr/bin/env python2.7

from nagios import BaseCheckCommand

try:
    from requests import HTTPError
    requests_installed = True
except ImportError:
    requests_installed = False

try:
    from sumologic import SumoLogic
    sumo_installed = True
except ImportError:
    sumo_installed = False

from optparse import OptionParser
import time
import datetime

class CollectorLastMessageCheckCommand(BaseCheckCommand):
    def __init__(self, **kwargs):
        # Exit early if prereqs are not installed
        self.prerequites_test()

        # Invoke parent class's constructor
        self.__class__.__bases__[0].__init__(self,\
                parser=OptionParser(usage='%prog [options] COLLECTOR_NAME', version="%prog 1.0"))

        # Constructor for CollectorLastMessage
        self.add_options()
        params      = kwargs
        opts, args  = self.parse_args()

        self.access_id      = params.get('access_id') or opts.access_id
        self.secret_key     = params.get('secret_key') or opts.secret_key
        self.collector_name = params.get('collector_name') or args[0]

        self.warning        = int(opts.warning)
        self.critical       = int(opts.critical)

        # Instantiating the session will not raise if there are invalid credentials
        self.session        = SumoLogic(self.access_id, self.secret_key)

    def check(self):
        if not self.collector_alive():
            # state is critical because the collector is dead as a doorknob
            self.status = "CRITICAL: %s is not alive!" % self.collector_name
            self.perf_data = "alive=0;1;1;;"
            self.exit(exit_code=self.CRITICAL_EXIT)

        time_diff = time.time() - self.last_message_time()
        if time_diff >= self.critical:
            # state is critical
            self.status = "CRITICAL: %s last logged %s seconds ago" % (self.collector_name, time_diff)
            self.perf_data = "last_message=%ss;%s;%s;; alive=1;1;1;;" % (time_diff, self.warning, self.critical)
            self.exit(exit_code=self.CRITICAL_EXIT)
        elif time_diff >= self.warning:
            # state is warning
            self.status = "WARNING: %s last logged %s seconds ago" % (self.collector_name, time_diff)
            self.perf_data = "last_message=%ss;%s;%s;; alive=1;1;1;;" % (time_diff, self.warning, self.critical)
            self.exit(exit_code=self.WARNING_EXIT)
        else:
            # state is OK
            self.status = "OK: %s last logged %s seconds ago" % (self.collector_name, time_diff)
            self.perf_data = "last_message=%ss;%s;%s;; alive=1;1;1;;" % (time_diff, self.warning, self.critical)
            self.exit()

    def add_options(self):
        self.parser.add_option('-a', '--access-id',
                                dest='access_id',
                                action='store',
                                help='Sets the SumoLogic API access ID to use for queries.',
                                metavar='ACCESS_ID')
        self.parser.add_option('-s', '--secret-key',
                                dest='secret_key',
                                action='store',
                                help='sets the SumoLogic API secret key to use for queries.',
                                metavar='SECRET_KEY')

    def prerequites_test(self):
        if not requests_installed:
            self.usage(UNKNOWN_EXIT,
                        msg='Unable to import requests. Ensure the "requests" package is installed. To install: `pip install requests`')

        if not sumo_installed:
            self.usage(UNKNOWN_EXIT, 
                        msg='Unable to import sumologic-sdk. Ensure the "sumologic-sdk" package is installed. To install: `pip install sumologic-sdk`')

    def collector_alive(self):
        try:
            collectors = self.session.collectors()
            for collector in collectors:
                if collector.get('name') == self.collector_name:
                    return collector.get('alive')
        except HTTPError as e:
            self.exit(err=e)

    def last_message_time(self):
        try:
            time_now = datetime.datetime.now()
            time_now_iso = time_now.isoformat().split('.')[0]
            time_twelve_hours_ago = (time_now - datetime.timedelta(hours=12)).isoformat().split('.')[0]
            search_res = self.session.search('_collector=%s | limit 1' % self.collector_name, fromTime=time_twelve_hours_ago, toTime=time_now_iso, timeZone='PST')
            if len(search_res) < 1:
                raise Exception("No messages found in search!")
            last_message = search_res.pop()
            print last_message.get('_messagetime')
            assert isinstance(last_message, dict),\
                    "Invalid API response."
        except Exception as e:
            self.exit(err=e)
        return int(last_message.get('_messagetime') / 1000)



# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open
import glob
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='omada-nagios',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='1.0.6',

    description='A framework for writing Nagios plugins',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/omadahealth/omada-nagios',

    # Author details
    author='Kevin Phillips',
    author_email='kevin.phillips@omadahealth.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: System Administrators',
        'Topic :: System :: Monitoring',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7'
    ],

    # What does your project relate to?
    keywords='omada-nagios nagios monitoring',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests', 'lib', 'include']),

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['sumologic-sdk','requests'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'dev': ['check-manifest', 'twine'],
        'test': ['mock'],
    },

    scripts=[
        'scripts/sumologic_collector-last-message'
    ],

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    #package_data={
    #    'sample': ['package_data.dat'],
    #},

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[
            ('/etc/nagios-plugins/config', glob.glob('definitions/*'))
    ],
)
